;===============================================================================
; MicrOS, a totally 80s, rudimentary OS.

; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details. 

; On the TI-84+CSE, performance margins are so far into the negative that there
; is never a reason to optimize for size over speed.  It's so bad I can't come
; up with an apt analogy.

; To do:
;  - Fix ScrollUp, which currently does nothing
;  - Switch to double-width font
;  - Editor:
;    - Find
;    - Options
;  - Need battery level testing!
;  - Proper exiting code; fix more port values.
;  - Some routines need to accept 32-bit pointers, not 16-bit
;    This may imply a lot of use of DEHL'
;  - Write test code!
;    - IncPtr, DecPtr, AddPtr, SubPtr

; Bugs:
;  - Interrupts consume like 10% of CPU time
;  + Hex editor help: fix F, XTthN, G, H
;  - Bug with disassembler not advancing correctly on prefix codes
;  + Screen gets bugged out if APD during screen I/O
;  + PORT MONITOR not centered

; - Fix Milo.  Needs to be streamlined.  Doesn't need to do wacky self-deletion!

; In this file:
;  - Compile time options
;  - To rebuild the INC file and the routines documentation, do build doc


;====== Equates ================================================================
.nolist
#include "convhex.inc"
.list

;#define ENABLE_FULL_ERASE TRUE ; This is pretty dangerous.

; Version
MicrOSVersion	.equ	23h + 0
#define MicrOSVersionString	chBeta, "2.3"

; Space-saving options
;#define HEX_ASCII_CHART TRUE
#define ENABLE_DISASM TRUE
;#define ENABLE_USB_SUPPORT TRUE
; I think I wanted to move some buffers like textBuffer to the same 768-byte
; zone that the other main variables are stored in; but I never actually did it.
;#define NO_EXT_RAM TRUE
#define ENABLE_EOS_QUIT
#define INCLUDE_HELP
#define IS_APPLICATION

#define number_of_pages 1

.echo	" "
.echo	"MICROS ASSEMBLY OPTIONS:"
#ifdef ENABLE_FULL_ERASE
	.echo	"Full chip erase enabled."
#endif
#ifdef HEX_ASCII_CHART
	.echo	"Hex editor ASCII charted included."
#endif
#ifdef ENABLE_USB_SUPPORT
	.echo	"USB support enabled"
ENABLE_USB_SUPPORT_VAL .equ	8
#else
ENABLE_USB_SUPPORT_VAL .equ	0
#endif
#ifdef ENABLE_DISASM
	.echo	"Disassembler enabled."
#endif

.echo	" "


#include "equates.asm"


;====== Header =================================================================
#ifdef IS_APPLICATION
	.org	4000h
	#include "appheader.asm"
#else
	.org	0C080h-2
	.db	0D9h, 3Fh ; Stop, Newline
#endif
starting_address:
#include "jumptable.asm"

#include "sysinit.asm"


;====== Includes ===============================================================

#include "unlockflash.asm"
#include "utility.asm"
#include "ramcode.asm"
#include "lcd.asm"
text_driver_start:
#include "textmode.asm"
text_driver_end:
port_mon_start:
#include "portmon.asm"
port_mon_end:
#include "lcdportmon.asm"
editor_start:
#include "hexeditor.asm"
editor_end:
memory_driver_start:
#include "memory.asm"
memory_driver_end:
#include "interrupts.asm"
keyboard_driver_start:
#include "keyboard.asm"
keyboard_driver_end:
#include "payload.asm"
;#ifdef ENABLE_USB_SUPPORT
;	#include "directusb.asm"
;#endif
#ifdef ENABLE_DISASM
disasm_code_start:
	#include "disassembler.asm"
disasm_code_end:
#endif
end_of_code_segment:

;#ifdef ENABLE_USB_SUPPORT
;	#include "directusb_data.asm"
;#endif
#ifdef ENABLE_DISASM
disasm_data_start:
	#include "disassembler_data.asm"
disasm_data_end:
#endif
#include "micros_data.asm"
#include "lcd_data.asm"
keyboard_driver_data_start:
#include "keyboard_data.asm"
keyboard_driver_data_end:
port_mon_data_start:
#include "portmon_data.asm"
port_mon_data_end:
#include "lcdportmon_data.asm"
editor_data_start:
#include "hexeditor_data.asm"
editor_data_end:
#include "payload_data.asm"
font_start:
#include "font_data.asm"
font_end:
#ifdef INCLUDE_HELP
	#include "help_data.asm"
#endif
end_of_data_segment:

;------ Information ------------------------------------------------------------
.echo	" "
.echo	" * INFORMATION & STATISTICS * "
#ifdef REGENERATE_DOCUMENTATION
 .echo	"Documentation rebuilt? YES"
#else
 .echo	"Documentation rebuilt? NO (use build doc or spasm -D REGENERATE_DOCUMENTATION)"
#endif
.echo	"Version word: ", convhex(MicrOSVersion)
#ifdef IS_OS
 .echo	"Number of entries in the jump table: ", (last_jump_table_entry_point - first_jump_table_entry_point) / 3
#endif
#ifdef IS_APPLICATION
 .echo	"Pages: ", number_of_pages
 .echo	"Number of exported routines: ", exportedRoutinesCount
#endif
.echo	"Location of static vars: ", convhex(start_of_static_vars)
.echo	"Size of static vars: ", size_of_static_vars, " bytes"
.echo	"Last address before bookmarks: ", convhex(editBookmarks);lastCurCol)
.echo	"End of static vars: ", convhex(end_of_static_vars)
.echo	"Last address of executable code: ", convhex(end_of_code_segment)
.echo	"Last address used by MicrOS: ", convhex(end_of_data_segment)
.echo	" * SIZES * "
#ifdef ENABLE_DISASM
 .echo	"Disassembler code size: ", disasm_code_end - disasm_code_start, " bytes"
 .echo	"Disassembler data size: ", disasm_data_end - disasm_data_start, " bytes"
 .echo	"Number of instruction strings: ", last_instr_string
#endif
#ifdef INCLUDE_HELP
 .echo	"Help size: ", help_end - help_start, " bytes"
#endif
.echo	"Text driver size: ", text_driver_end - text_driver_start, " bytes"
.echo	"Font size: ", font_end - font_start, " bytes"
.echo	"Port monitor size: ", port_mon_end + port_mon_data_end - port_mon_start - port_mon_data_start, " bytes"
.echo	"Hex editor size: ", editor_end + editor_data_end - editor_start - editor_data_start, " bytes"
.echo	"Memory driver size: ", memory_driver_end - memory_driver_start, " bytes"
.echo	"Keyboard driver size: ", keyboard_driver_end + keyboard_driver_data_end - keyboard_driver_start - keyboard_driver_data_start, " bytes"
bah .equ sysCallSize+sysJumpSize+pagedCallSize+(ldDEHLIndEnd-LdDEHLIndSource)+(multHByEEnd-MultHByESource)+(realIsrEnd-realIsrSource)+(putByteEnd-PutByteSource)
bah2 .equ (getByteEnd-GetByteSource)+(subPtrEnd-SubPtrSource)+(addPtrEnd-AddPtrSource)+(decPtrEnd-DecPtrSource)+(incPtrEnd-IncPtrSource)+(getByteIncPtrEnd-GetByteIncPtrSource)
bah3 .equ bah + bah2
.echo	"Size of RAM routines: ", bah3, " bytes"
.echo	"Total code segment size: ", end_of_code_segment - starting_address, " bytes"
.echo	"Total data segment size: ", end_of_data_segment - end_of_code_segment, " bytes"
.echo	"Total MicrOS size: ", end_of_data_segment - starting_address, " bytes (sans signature)"
#ifdef IS_APPLICATION
 .echo	"Page 1 space remaining: ", 16317 - (end_of_data_segment - starting_address), " bytes"
#endif

.echo	" "


;------ Routines ---------------------------------------------------------------
#ifndef IS_APPLICATION
	.error	"Meh, never bothered with fixing the RAM program version."
#endif


.end
.end
