; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

ramCodeLoadingTable:
; You shouldn't reorder these.  It will mess things up.
	.dw	SysCallSource-2
	.dw	SysJumpSource-2
	.dw	PagedCallSource-2
	.dw	dummyRamCodeEntry
	.dw	LdDEHLIndSource-2
	.dw	MultHByESource-2
	.dw	realIsrSource-2
	.dw	PutByteSource-2
	.dw	GetByteSource-2
	.dw	SubPtrSource-2
	.dw	AddPtrSource-2
	.dw	DecPtrSource-2
	.dw	IncPtrSource-2
	.dw	GetByteIncPtrSource-2
ramCodeLoadingTableEnd:
	.dw	0

#if ((ramCodeLoadingTableEnd-ramCodeLoadingTable)/2) != numberOfRamRoutines
	.error	"Fix numberOfRamRoutines in equates.asm!"
#endif
;numberOfRamRoutines	.equ	(ramCodeLoadingTableEnd-ramCodeLoadingTable)/2

dummyRamCodeEntry:
	.dw	1
	.db	0

;------ SysCall ----------------------------------------------------------------
#define	SysCallSize	(sysCallEnd-SysCallSource)
	.dw	SysCallSize
SysCallSource:
documentRoutine("SysCall")
document("Performs a MicrOS system call.")
document("Kept in RAM to ensure that it is always available.")
document("Input:")
document(" - IY: System routine page+address")
document("Output:")
document(" - See routine documentation")
document("Destroys:")
document(" - IY")
document(" - AF'")
	ex	af, af'					; 4	1
	in	a, (pMPgA)				; 11	2
	push	af					; 11	1
	in	a, (pMPgAHigh)				; 11	2
	push	af					; 11	1
	push	hl					; 11	1
	ld	hl, (microsBasePage)			; 16	3
	; We want to do hl - a . . . a = l - a  ->  a = l + -a
	ld	a, iyh					; 8	2
	rlca						; 4	1
	rlca						; 4	1
	and	3					; 7	2
	neg						; 8	2
	add	a, l					; 4	1
	jp	p, +_-SysCallSource+SysCallLoc		; 10	3
	cp	90h					; - 7	2
	jr	c, +_					; - 12	3
	sub	80h					; - 7	2
	dec	h					; - 4	1
_:	out	(pMPgA), a				; 11	2
	ld	a, h					; 4	1
	out	(pMPgAHigh), a				; 11	2
	ld	a, iyh					; 4	2
	and	3Fh					; 7	2
	or	40h					; 7	2
	ld	iyh, a					; 8	2
	ex	af, af'					; 4	1
	ld	hl, retPoint-SysCallSource+SysCallLoc	; 10	3
	ex	(sp), hl				; 19	1
	jp	(iy)					; 10	2
retPoint:
	ex	af, af'					; 4	1
	pop	af					; 11	1
	out	(pMPgAHigh), a				; 11	2
	pop	af					; 11	1
	out	(pMPgA), a				; 11	2
	ex	af, af'					; 4	1
	ret						; 10	1
sysCallEnd:
							; 292 cc


;------ SysJump ----------------------------------------------------------------
#define	SysJumpSize	(sysJumpEnd-SysJumpSource)
SysJumpLoc	.equ	SysCallLoc+SysCallSize
	.dw	SysJumpSize
SysJumpSource:
documentRoutine("SysJump")
document("Performs a MicrOS system call")
document("Kept in RAM because any other solution would be too ugly.")
document("Input:")
document(" - IY: System routine page+address")
document("Output:")
document(" - See routine documentation")
document("Destroys:")
document(" - AF'")
document(" - BC'")
document(" - DE'")
document(" - HL'")
document(" - IY")
	exx
	ex	af, af'
	push	iy
	pop	hl
	ld	a, h
	res	7, h
	set	6, h
	and	0C0h
	ld	b, a
	ld	c, 0
	ld	de, (microsBasePage)
	call	SubPtr
	ld	a, e
	out	(pMPgA), a
	ld	a, d
	out	(pMPgAHigh), a
	push	hl
	ex	af, af'
	exx
	ret
sysJumpEnd:


;------ PagedCall --------------------------------------------------------------
PagedCallLoc	.equ	SysJumpLoc+SysJumpSize
#define	PagedCallSize	(pagedCallEnd-PagedCallSource)
	.dw	PagedCallSize
PagedCallSource:
documentRoutine("PagedCall")
document("Calls the routine at DEHL.  You're on your own to figure out how to pass")
document("parameters.")
document("Kept in RAM because I was too lazy to work up a solution using syscall.")
document("Input:")
document(" - DEHL: Pointer to routine")
document("Output:")
document(" - Routine called")
document("Destroys:")
document(" - AF'")
	ex	af, af'
	pushPageA()
	ex	af, af'
	push	hl
	ld	hl, +_-PagedCallSource+PagedCallLoc
	ex	(sp), hl
PagedJumpSource:
documentRoutine("PagedJump")
document("This is just like PagedCall, except that it skips the part about pushing the")
document("current page and return address onto the stack.  This routine literally jumps")
document("into the middle of PagedCall.")
	ex	af, af'
	ld	a, e
	out	(pMPgA), a
	ld	a, d
	out	(pMPgAHigh), a
	ex	af, af'
	jp	(hl)
_:	ex	af, af'
	popPageA()
	ex	af, af'
	ret
pagedCallEnd:


;------ InitRamRoutines --------------------------------------------------------
InitRamRoutines:
documentRoutine("InitRamRoutines")
document("Initalizes routines that are copied to RAM staticly.")
document("Inputs:")
document(" - None")
document("Outputs:")
document(" - None")
document("Destroys:")
document(" - Assume all")
	ld	ix, ramCodeLoadingTable
	ld	iy, ramCodeJumpTable
	ld	de, ramCodeLoadingLocation
;	.echo	"ramCodeLoadingTable: ", convHex(ramCodeLoadingTable)
;	.echo	"ramCodeJumpTable: ", convHex(ramCodeJumpTable)
;	.echo	"ramCodeLoadingLocation: ", convHex(ramCodeLoadingLocation)
;	.echo	"SysCallLoc", SysCallLoc
;	.echo	"ramCodeLoadingLocation", ramCodeLoadingLocation
;	ld	a, numberOfRamRoutines
_:	ld	l, (ix)
	inc	ix
	ld	h, (ix)
	inc	ix
	ld	a, h
	or	l
	jr	z, +_
	ld	c, (hl)
	inc	hl
	ld	b, (hl)
	inc	hl
	ld	(iy), 0C3h	; JP
	inc	iy
	ld	(iy), e
	inc	iy
	ld	(iy), d
	inc	iy
	ldir
;	dec	a
;	jr	nz, -_
	jr	-_
_:	ld	de, PagedCallLoc+PagedJumpSource-PagedCallSource
	ld	(PagedJump+1), de
	ret


;------ ExecRamCode ------------------------------------------------------------
ExecRamCode:
#define sysExecRamCode .error "ExecRamCode can only load code from MicrOS page 0!"
; Copies the code block pointed to by IY into RAM and then runs it.
; The code block must be prefixed with a word specifing its size.
; Input:
;  - IY: Pointer to code block
; Output:
;  - Whatever the routine outputs
; Destroys:
;  - Nothing
	push	hl
	push	de
	push	bc
	ld	c, (iy)
	inc	iy
	ld	b, (iy)
	inc	iy
	ld	de, microsRamCode
	push	iy
	pop	hl
	ldir
	pop	bc
	pop	de
	pop	hl
	jp	microsRamCode


