; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

documentSection("SYSTEM INITIALIZATION AND TERMINATION")
StartMicrOS:
#ifndef IS_APPLICATION
	; Boot with HL = return address
	ld	(quitaddr), hl
	ld	(spaddr), sp
	in	a, (pMPgA)
	ld	(pageAOrig), a
	in	a, (pMPgAHigh)
	ld	(pageAOrig+1), a
	in	a, (pMPgB)
	ld	(pageBOrig), a
	in	a, (pMPgBHigh)
	ld	(pageBOrig+1), a
;	jp	RebootMicrOS
#else
	call	UnlockFlash
#endif

;====== Init ===================================================================

RebootMicrOS:
documentRoutine("RebootMicrOS")
document("Initalize Micros")
initMOS:
	di
	ld	sp, topOfStack
	xor	a
	out	(pMemDelay), a
	inc	a	;ld	a, cpu15MHz
	out	(pCpuSpeed), a
	ld	a, crstlClockKeepAlive | crstlTmrKeepAlive
	out	(pCrstlKeepAlive), a
	ld	a, delayCycleMin
	out	(pLcdDelayCpu6MHz), a
	out	(pLcdDelayCpu15MHz), a
	out	(pLcdDelayCpu20MHz), a
	out	(pLcdDelayCpu25MHz), a
	; Memory map
	xor	a
	out	(pFlashExecLowerLimit), a
	out	(pFlashExecUpperLimit), a
	out	(pRamExecLowerLimit), a
	out	(pBlockMRemapC), a
	out	(pBlockMRemapB), a
	dec	a	; ld a, 0FFh
	out	(pRamExecUpperLimit), a
	out	(pFlashExecHighBit), a
	ld	a, flashType4MB; | ramType256K;ramType128K
	out	(pFlashRamType), a
	; GPIO
	ld	a, 0F0h
	out	(pGpioDirection), a
	; Clear variables
	ld	sp, topOfStack
	ld	hl, start_of_static_vars
	ld	b, size_of_static_vars
	xor	a
	call	ClearMem
	call	SetUpInterrupts
	in	a, (pMPgA)
	ld	e, a
	in	a, (pMPgAHigh)
	ld	d, a
	ld	(microsBasePage), de
	; Initalize all these to the same value until this actually has more than one page.
;	call	++_
	ld	(microsBasePage+2), de
;	call	++_
	ld	(microsBasePage+4), de
;	call	++_
	ld	(microsBasePage+6), de
;	jr	+_
;_:	dec	e
;	ret	p
;	ld	e, 7Fh
;	dec	d
;	ret
_:	ld	bc, cursorTimerDefaultValue
	ld	(cursorTimerPeriod), bc
	ld	(cursorTimer), bc
	ld	a, kbdScanDivisor
	ld	(kbdScanTimer), a
	ld	bc, keyBlockTime*256 + minKeyHoldTime
	ld	(lastKeyHoldTimer), bc
	ld	bc, keyRepeatWaitTime*256 + keyFirstRepeatWaitTime
	ld	(lastKeyFirstRepeatTimer), bc
	; Brightness
	ld	a, defaultBrightness
	ld	(brightness), a
#ifdef IS_APPLICATION
	call	InitRamRoutines
#endif	
	; inverse mode: unset this
	ei	; Not done by SetUpInterrupts
	call	BacklightSetLevel
MainMenu:
; A syscall for this is generated by the jump table file.
	ld	sp, topOfStack
	ld	a, apdEnabledMask
	ld	(mflags + appFlags), a
	xor	a
	ld	(mflags + textFlags), a
	ld	(mflags + editorFlags), a
	ld	(mflags + curFlags), a
	call	ClrScrnFull
	call	ResetScreen
	ld	hl, mMain
	ld	de, (microsPage0)
	jp	Menu


;====== Reboot =================================================================
RebootDevice:
documentRoutine("RebootDevice")
document("Attempts to hard reboot the device.")
	xor	a
	out	(pRamExecLowerLimit), a
	out	(pRamExecUpperLimit), a
	; We're STILL not crashed!?
	out	(pFlashExecLowerLimit), a
	dec	a
	out	(pFlashExecUpperLimit), a
	out	(pMPgB), a
	out	(pMPgBHigh), a
	ld	a, flashExecUpperLimitHigh
	out	(pFlashExecHighBit), a
	; If we're still not crashed, something super-funky is going on.
	jp	8000h


;====== Suspend ================================================================
Suspend:
	ld	hl, MainMenu
	push	hl
MicrosApd:
documentRoutine("MicrosApd")
document("Suspends MicrOS, and then returns to the caller.")
	push	af
	push	bc
	push	de
	push	hl
	push	ix
	call	LcdOff
	in	a, (pIntMask)
	push	af
	ld	a, intOnKey | intLinkport; | intDisableLowPowerMode; and NOT intDisableLowPowerMode
	;ld	a, intTmr2 | intOnKey | intDisableLowPowerMode
	out	(pIntMask), a
	ei
	halt
	pop	af
	out	(pIntMask), a
	call	LcdInit
;	call	SetBrightness
	call	SetDirectionDown
	call	SetLcdWindowColumnBounds
	call	FixCursor
	call	GetCSC
	pop	ix
	pop	hl
	pop	de
	pop	bc
	pop	af
	ret
	
	


;====== Quit ===================================================================
QuitToEOS:
#ifdef ENABLE_EOS_QUIT
	di
	; Reset screen
	xor	a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
;	setLcdReg(lrWinLeft, 0, 0)
;	setLcdReg(lrWinRight, lcdHigh(lcdWidth-1), lcdLow(lcdWidth-1))
;	setLcdReg(lrWinTop, 0, 0)
;	setLcdReg(lrWinBottom, lcdHigh(lcdHeight-1), lcdLow(lcdHeight-1))
;	setLcdDReg(lrEntryMode, lcdDispCtrlDefault)
	ld	ix, lcdReadyForEOS
	call	ExecScrnCmds
#ifndef IS_APPLICATION
	ld	sp, (spaddr)
	ld	a, (pageAOrig)
	out	(pMPgA), a
	ld	a, (pageAOrig+1)
	out	(pMPgAHigh), a
	ld	a, (pageBOrig)
	out	(pMPgB), a
	ld	a, (pageBOrig+1)
	out	(pMPgBHigh), a
	ld	hl, (quitaddr)
	jp	(hl)
#else
	ld	sp, 0FFF0h	; Eh, JForceCmdNoChar will fix the stack.
	ld	iy, 8B26h; flags
	set	 0, (iy+12h)	;set	indicRun, (iy+shiftFlags);indicFlags)
	res	 5, (iy+0)	;res	DonePrgm, (iy+DoneFlags)
	res	 4, (iy+9)	;res	OnInterrupt, (iy+OnFlags)
	set	 2, (iy+0Dh)	;set	appAutoScroll, (iy+appFlags)
	res	 4, (iy+0Dh)	;res	appLockMenus, (iy+appFlags)
	; Interrupts and stuff?
	ld	a, 81h
	out	(pMPgB), a
	dec	a
	out	(pMPgC), a
	ld	a, 1011b
	out	(pIntMask), a
	im	1
	ei
	rst	28h	;b_call(_DrawStatusBarInfo)
	.dw	555Ah
	rst	28h	;b_call(_DrawStatusBar)
	.dw	54F4h
	rst	28h	;b_call(_maybe_ClrScrn)
	.dw	4540h
	rst	28h
	.dw	4027h	;_JForceCmdNoChar
#endif
#else
	; Uh ... just boot whatever OS is there, I guess
	call	ClrScrnFull
	jp	0053h
#endif


;====== Support Routines =======================================================
;------ Panic Handler ----------------------------------------------------------
Panic:
documentRoutine("Panic")
document("This is supposed to show an error message and then reinitalize everything.")
	pop	hl
	ld	sp, topOfStack
	push	hl
	call	ResetScreen
	ld	hl, strPanic
	call	PutS
	pop	hl
	call	DispHL
_:	call	RawGetCSC
	or	a
	jr	nz, -_
	jp	RebootMicrOS
	

;------ Get Version ------------------------------------------------------------
GetVersion:
documentRoutine("GetVersion")
document("This is supposed to return information about MicrOS.");
	ld	hl, MicrOSVersion
	;ld	a, IS_OS_VAL + RUN_FROM_FLASH_VAL + EDITOR_8000h_VAL + ENABLE_USB_SUPPORT_VAL + IS_APP_VAL
	ret

