; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

documentSection("TEXT DRIVER")
document("These routines are all related to the text-mode driver.")
document("See arch.txt for more information about the text API.")
document("To do: Fix ScrollUp, which currently does nothing.")

;Memory: currentRow, currentCol
;windLeft, windRight, windTop, windBottom
;tab1, tab2, tab3, tab4
;Flags: Inverse, scroll enable, scroll happened
;Functions:
;+PutC
;+ClrScrnFull
;+PutS, +ClearWind, +PutSpace, +PutSpaces, +PutSClrWind
;+HomeUp, +NewLine, +ScrollUp, +NewLineClrEOL
;+Locate, +NextTab, +CursorRight, +CursorLeft, +CursorUp, +CursorDown, +AdvanceCursor, +CursorBack
;+CursorForward
;SetWind, +SetFullScrnWind, +ClearWind
;CenterText
;DrawBox, DrawBanner, DrawBoxTitle
;+ReadChar
;WriteSideChan, ReadSideChan
;+SetDirectionDown
;+SetDirectionRight
;+SetCharRow, +SetCharCol, +SetLcdWindowColumnBounds, +FixCursor, +ResetScreen, NormalizeScreen
;+InitScreenModes
;+FillScrnFull


; To do: Fix New line, putsclrwind, make sure ScrollUp won't infinite loop on the last character of PutSpaces
; And yeah.
; Also, rework everything for double-size 5x7 characters.
; And write a test program for the double-width mode.
; And ScrollUp needs to pay attention to the high bit of the read character.


;------ ScrollUp ---------------------------------------------------------------
ScrollUp:
documentRoutine("ScrollUp")
document("Scrolls the window up and erases the bottom-most line, leaving the cursor at")
document("the left side of the line.")
document("Well, it's supposed to, anyway.  I need to figure out how to prevent this from")
document("having infinite recursion issues.")
document("Input:")
document(" - Character codes in LCD pixel subcode")
document("Output:")
document(" - Screen scrolled")
document("Destroys:")
document(" - Any chance of reasonable performance")
document(" - AF.  Why would you call this when you need to save AF?")
document(" - B")
	
	ret
	
	push	hl
	ld	a, (windBottom)
	ld	h, a
	ld	a, (windTop)
	sub	h
	jr	c, scrollUpSpaces
	jr	z, scrollUpSpaces
	ld	a, (windRight)
	ld	(currentCol), a
	ld	a, (windTop)
	ld	l, a
_:	ld	a, (currentCol)
	ld	h, a
	inc	l
	call	Locate
	call	ReadChar
	dec	l
	call	Locate
	call	PutC
	ld	a, (windRight)
	cp	h
	jr	nz, -_
	ld	a, (windRight)
	ld	(currentCol), a
	inc	l
	ld	a, (windBottom)
	cp	l
	jr	nz, -_
scrollUpSpaces:
	ld	a, (windLeft)
	ld	h, a
	ld	a, (windRight)
	sub	h
	inc	a
	ld	b, a
	call	PutSpaces
	ld	a, (windLeft)
	ld	h, a
	ld	a, (windBottom)
	ld	a, l
	call	Locate
	pop	hl
	ret


	


;------ SetFullScrnWind --------------------------------------------------------
SetFullScrnWind:
documentRoutine("SetFullScrnWind")
document("Resets the window to one in which everything is full-screen")
document("Inputs:")
document(" - None")
document("Output:")
document(" - Window set")
document("Destroys:")
document(" - BC")
	ld	bc, 0
	ld	(windTop), bc
	ld	bc, (textCols) * 256 + textRows
	ld	(windBottom), bc
	ret


;------ ClearWind --------------------------------------------------------------
ClearWind:
documentRoutine("ClearWind")
document("Clears the current window by printing lots of spaces.")
document("Input:")
document(" - Window bounds")
document("Output:")
document(" - Window cleared and cursor moved to top left")
document("Destroys:")
document(" - Assume everything")
	call	HomeUp
_:	call	NewLineClrEOL
	ld	a, (windTop)
	ld	b, a
	ld	a, (currentRow)
	cp	b
	jr	nz, -_
	jp	HomeUp


;------ PutSClrWind ------------------------------------------------------------
PutSClrWind:
documentRoutine("PutSClrWind")
document("Displays a string.  If the string contains control codes, those codes are")
document("parsed.  New lines will print spaces until the end of the line.")
document("Input:")
document(" - DEHL: String to show")
document("Output:")
document(" - String shown")
document(" - DEHL advanced to the byte after the null terminator.")
document("Destroys:")
document(" - AF")
	push	bc
putSClrWindLoop:
	call	GetByte	; ld	a, (hl)
	push	af
	call	IncPtr	; inc	hl
	pop	af
	cp	ch1stPrintableChar
	call	nc, PutC2
	jr	nc, putSClrWindLoop
	or	a
	jr	z, putSClrWindFillToEnd
	cp	chNewLine
	jr	nz, +_
	call	NewLineClrEOL
	jr	putSClrWindLoop
_:	cp	chTab
	call	z, NextTab
	jr	putSClrWindLoop	; Just display spaces until the last column of the last line
putSClrWindFillToEnd:
	push	de
	push	hl
	ld	a, (windTop)
	ld	b, a
	ld	a, (currentRow)
	cp	b
	jr	nz, +_
	ld	a, (windLeft)
	ld	b, a
	ld	a, (currentCol)
	cp	b
	jr	z, putSClrWindRet
_:	call	NewLineClrEOL
	ld	a, (windTop)
	ld	b, a
	ld	a, (currentRow)
	cp	b
	jr	nz, -_
putSClrWindRet:
	call	HomeUp
	pop	hl
	pop	de
	pop	bc
	ret


;------ PutS -------------------------------------------------------------------
PutS:
documentRoutine("PutS")
document("Displays a string.  If the string contains control codes, those codes are")
document("parsed.")
document("Input:")
document(" - DEHL: String to show")
document("Output:")
document(" - String shown")
document(" - DEHL advanced to the byte after the null terminator.")
document("Destroys:")
document(" - AF")
	call	GetByte	; ld	a, (hl)
	push	af
	call	IncPtr	; inc	hl
	pop	af
	cp	ch1stPrintableChar
	call	nc, PutC2
	jr	nc, PutS
	or	a
	ret	z
	push	bc
	push	de
	push	hl
	cp	chNewLine
	jr	nz, +_
	call	NewLine
	jr	++_
_:	cp	chTab
	call	z, NextTab
_:	pop	hl
	pop	de
	pop	bc
	jr	PutS


;------ Locate -----------------------------------------------------------------
Locate:
documentRoutine("Locate")
document("Moves the cursor to a specific location.")
document("Inputs:")
document(" - H: Column")
document(" - L: Row")
document("Output:")
document(" - Cursor moved")
document("Destroys:")
document(" - Nothing")
	ld	(currentRow), hl
	push	af
	push	bc
	push	de
	push	hl
	call	SetCharRow
	call	SetCharCol
	pop	hl
	pop	de
	pop	bc
	pop	af
	ret


;------ HomeUp -----------------------------------------------------------------
HomeUp:
documentRoutine("HomeUp")
document("Moves the text cursor up to the top, left of the text window.")
document("Inputs:")
document(" - None")
document("Output:")
document(" - Cursor moved")
document("Destroys:")
document(" - A")
	push	hl
	ld	a, (windTop)
	ld	l, a
	ld	a, (windLeft)
	ld	h, a
	call	Locate
	pop	hl
	ret

	
;------ CursorBack -------------------------------------------------------------
CursorBack:
documentRoutine("CursorBack")
document("Moves the cursor back one unit.  May move to the rightmost position of the")
document("previous line.  May also move to bottom-right of window if cursor was")
document("previously at top-left.")
document("Input:")
document(" - Current cursor location")
document("Output:")
document(" - Cursor moved.")
document("Destroys:")
document(" - AF")
document(" - C")
	ld	a, (currentCol)
	or	a
	jr	+_
	ld	c, a
	ld	a, (windLeft)
	cp	c
	jr	z, +_
	ld	a, c
	dec	a
	ld	(currentCol), a
	jp	SetCharCol
_:	ld	a, (windRight)
	ld	(currentCol), a
	ld	a, (currentRow)
	or	a
	ret	z
	ld	c, a
	ld	a, (windTop)
	cp	c
	;ret	z
	jr	nz, +_
	ld	a, (windBottom)
	ld	c, a
_:	ld	a, c
	dec	a
	ld	(currentRow), a
	jp	FixCursor


;------ CursorForward ----------------------------------------------------------
CursorForward:
documentRoutine("CursorForward")
document("Moves the cursor forward one unit.")
document("Input:")
document(" - Current cursor location")
document("Output:")
document(" - Cursor moved.")
document("Destroys:")
document(" - Whatever AdvanceCursor destorys.")
document(" - AF")
	call	AdvanceCursor
	jp	FixCursor


;------ CursorRight ------------------------------------------------------------
CursorRight:
documentRoutine("CursorRight")
document("Moves the cursor right, but does not allow the cursor to move past the right-")
document("most edge of the window.")
document("Input:")
document(" - Current cursor location")
document("Output:")
document(" - Cursor moved.")
document("Destroys:")
document(" - AF")
document(" - C")
document(" - HL")
	ld	a, (windRight)
	ld	c, a
	ld	a, (currentCol)
	inc	a
	cp	c
	ret	nc
	ld	(currentCol), a
	jp	SetCharCol


;------ CursorLeft -------------------------------------------------------------
CursorLeft:
documentRoutine("CursorLeft")
document("Moves the cursor left, but does not allow the cursor to move past the left-")
document("most edge of the window.")
document("Input:")
document(" - Current cursor location")
document("Output:")
document(" - Cursor moved.")
document("Destroys:")
document(" - AF")
document(" - C")
document(" - HL")
	ld	a, (currentCol)
	dec	a
	ld	c, a
	ld	a, (windLeft)
	cp	c
	ret	nc
	ld	(currentCol), a
	jp	SetCharCol


;------ CursorDown -------------------------------------------------------------
CursorDown:
documentRoutine("CursorDown")
document("Moves the cursor down one row, but won't move down past the bottom of the")
document("window.")
document("Input:")
document(" - Current cursor location")
document("Output:")
document(" - Cursor moved.")
document("Destroys:")
document(" - AF")
document(" - BC")
document(" - E")
	ld	a, (windBottom)
	ld	c, a
	ld	a, (currentRow)
	inc	a
	cp	c
	ret	nc
	ld	(currentRow), a
	jp	SetCharRow
	
;------ CursorUp ---------------------------------------------------------------
CursorUp:
documentRoutine("CursorUp")
document("Moves the cursor upone row, but won't move down past the top of the window.")
document("Input:")
document(" - Current cursor location")
document("Output:")
document(" - Cursor moved.")
document("Destroys:")
document(" - AF")
document(" - BC")
document(" - E")
	ld	a, (currentRow)
	dec	a
	ld	c, a
	ld	a, (windTop)
	cp	c
	ret	nc
	ld	(currentRow), a
	jp	SetCharRow


;------ AdvanceCursor ----------------------------------------------------------
AdvanceCursor:
documentRoutine("AdvanceCursor")
document("After a function like PutSpace or PutC has draw the character, this updates")
document("the text cursor to match.  This may trigger advancing to a new line.")
document("It may also trigger scrolling, which will really slow things down.")
document("Input:")
document(" - textAutoScroll, ")(iy + textFlags)
document("Output:")
document(" - Cursor advanced.")
document("Destroys:")
document(" - A")
document(" - B")
	ld	a, (currentCol)
	inc	a
	ld	(currentCol), a
	ld	b, a
	ld	a, (windRight)
	cp	b
	jr	z, NewLine
	ret	nc


;------ NewLine ----------------------------------------------------------------
document("Advances the cursor to the next line.")
document("Inputs:")
document(" - None")
document("Output:")
document(" - Cursor moved.")
document("Destroys:")
document(" - A")
document(" - B")
NewLine:
	;ld	a, (mflags + textFlags)
	;bit	textNewlineClr, a
	;jp	nz, NewLineClrEOL
	ld	a, (windLeft)
	ld	(currentCol), a
	ld	a, (currentRow)
	inc	a
	ld	(currentRow), a
	ld	b, a
	ld	a, (windBottom)
	cp	b
	;jr	nc, +_
	jr	z, +_
	jp	nc, FixCursor
_:	ld	a, (windTop)
	ld	(currentRow), a
;	ld	a, (mflags + textFlags)
;	bit	textAutoScroll, a
;	jp	nz, ScrollUp
	jp	FixCursor


;------ NewLineClrEOL ----------------------------------------------------------
NewLineClrEOL:
documentRoutine("NewLineClrEOL")
document("Moves the cursor to the next line, clearing the remainder of the current line.")
document("Inputs:")
document(" - None")
document("Output:")
document(" - Cursor moved")
document("Destroys:")
document(" - AF")
document(" - BC")
	push	hl
	push	de
_:	call	PutSpace
;	ld	a, 05Fh
;	call	PutC
	ld	a, (currentCol)
	ld	b, a
	ld	a, (windLeft)
	cp	b
	jr	nz, -_
	pop	de
	pop	hl
	ret


;------ NewLine2 ---------------------------------------------------------------
NewLine2:
documentRoutine("NewLine2")
document("Moves the cursor to the next line, unless it is already at the start of a new")
document("line.  Useful if you want to display data all the way up to the last column.")
	ld	a, (currentCol)
	ld	b, a
	ld	a, (windLeft)
	cp	b
	jp	nz, NewLine
	ret


;------ NewLineClrEOL2 ---------------------------------------------------------
NewLineClrEOL2:
documentRoutine("NewLineClrEOL2")
document("Moves the cursor to the next line, clearing the remainder of the current line,")
document("unless the cursor is already at the start of a new line.  ")
document("Useful if you want to display data all the way up to the last column.")
document("Inputs:")
document(" - None")
document("Output:")
document(" - Cursor moved")
document("Destroys:")
document(" - AF")
document(" - BC")
	ld	a, (currentCol)
	ld	b, a
	ld	a, (windLeft)
	cp	b
	jp	nz, NewLineClrEOL
	ret


;------ NextTab ----------------------------------------------------------------
NextTab:
documentRoutine("NextTab")
document("Moves the cursor forward to the next tab position.")
document("Inputs:")
document(" - Tab array")
document(" - currentCol")
document("Output:")
document(" - Cursor advanced")
document("Destroys:")
document(" - Nothing")
	push	af
	push	bc
	push	de
	push	hl
	ld	hl, currentCol
	ld	a, (hl)
	inc	hl
	ld	b, (hl)
	inc	hl
	cp	b
	jr	c, +_
	ld	b, (hl)
	inc	hl
	cp	b
	jr	c, +_
	ld	b, (hl)
	inc	hl
	cp	b
	jr	c, +_
	ld	b, (hl)
	inc	hl
	cp	b
	jr	nc, tabRestoreRegisters
_:	ld	c, a
	ld	a, b
	sub	c
	ld	b, a
	call	PutSpaces
tabRestoreRegisters:
	pop	hl
	pop	de
	pop	bc
	pop	af
	ret


;------ PutSpace ---------------------------------------------------------------
PutSpace:
documentRoutine("PutSpace")
document("Draws a single blank cell to the screen.")
document("Input:")
document(" - textInverse, ")(iy + textFlags)
document("Output:")
document(" - Spaces written")
document("Destroys:")
document(" - A")
document(" - B")
document("Assumes:")
document(" - LCD window already set")
document(" - LCD write cursor already set")
	; Tell it we want to write pixels
	ld	a, lrGram
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	ld	b, (charWidth * charHeight) / 2
	ld	a, (mflags + textFlags)
	bit	textInverse, a
	ld	a, 0	; Do not change to XOR A
	jr	z, +_
	dec	a
_:	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	djnz	-_
	jp	AdvanceCursor


;------ PutSpaces --------------------------------------------------------------
PutSpaces:
documentRoutine("PutSpaces")
document("Sends several spaces to the screen.  This routine may cause scrolling if")
document("scrolling is enabled.")
document("Input:")
document(" - B: Number of space to display")
document("Output:")
document(" - Cursor moved, space erased.")
document("Destroys:")
document(" - AF")
document(" - B")
	push	bc
	call	PutSpace
	pop	bc
	djnz	PutSpaces
	ret


;------ PutC2 ------------------------------------------------------------------
PutC2:
documentRoutine("PutC2")
document("Displays the character given, just like PutC.  However, this checks bit 7 to")
document("see whether the character should be displayed in inverse mode or not.")
document("If inverse mode is set in textFlags, then the meaning of bit 7 is reversed;")
document("that is, it inverts the inversion.")
document("Inputs:")
document(" - A: Character")
document(" - 7,A: Set for inverse mode.")
	push	af
	push	bc
	ld	b, a
	ld	a, (mflags + textFlags)
	push	af
	ld	c, a
	and	textInverseMask^255
	bit	textInverse, c
	ld	a, 0
	jr	z, +_
	xor	textInverseMask
_:	bit	7, b
	jr	z, +_
	xor	textInverseMask
	res	7, b
_:	or	c
	ld	(mflags + textFlags), a
	ld	a, b
	call	PutC	
	pop	af
	ld	(mflags + textFlags), a
	pop	bc
	pop	af
	ret


;------ PutC -------------------------------------------------------------------
PutC:
documentRoutine("PutC")
document("Draws a character to the screen.")
document("Input:")
document(" - A")
document("Output:")
document(" - Character drawn")
document(" - Cursor advanced to next location")
document("Destroys:")
document(" - Nothing")
document("Assumes:")
document(" - LCD window already set")
document(" - LCD write cursor already set")
; If you want to add color support, the best way to do that is probably to have
; H and L hold the foreground and background colors (assuming you don't feel
; the need for full 16-bit color in your text) and make IX the data pointer.
	push	af
	push	bc
	push	de
	push	hl
	ld	c, a
	pushAndDisableInterrupts
	xor	a
	; Sync
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	; Tell it we want to write pixels
	ld	a, lrGram
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	; If APD while displaying something, the screen will get confused
	ld	a, c
;	sub	minChar	; Not all 256 code points have bitmaps
;	jr	nc, +_
;	xor	a
;_:	; Multiply by 16
	ld	h, 0
	ld	l, a
	add	hl, hl
	add	hl, hl
	add	hl, hl
	add	hl, hl
	ld	de, fontData
	add	hl, de
; Inital output: Write ASCII code
	; Set bit 7 in ASCII if inverse text is set
	ld	e, a
	ld	a, (mflags + textFlags)
	ld	d, a
	ld	a, e
	bit	textInverse, d
	ld	d, 0
	jr	z, +_
	or	80h
	ld	c, a
	dec	d
_:	ld	a, (hl)
	inc	hl
	;ld	b, 8
	xor	d	; inverse text support
	ld	e, a
;_:	; Output byte loop
#macro PutCInitialOutput
	rrc	e
	sbc	a, a
	out	(pLcdData), a
	rrc	c
	jr	nc, +_
	xor	20h
_:	out	(pLcdData), a
#endmacro
;	djnz	--_
	; Let's optimize with loop unrolling!
	PutCInitialOutput
	PutCInitialOutput
	PutCInitialOutput
	PutCInitialOutput
	PutCInitialOutput
	PutCInitialOutput
	PutCInitialOutput
	PutCInitialOutput
; Main output loop
; B = bit loop counter
; C = byte loop counter
; D = inverse text flag
; E = text bitmap
; HL = ptr
;	ld	c, charLength-2
	ld	b, charLength-2
PutMapOutLoop:
	ld	a, (hl)
	inc	hl
;	ld	b, 8
	xor	d	; inverse text support
	ld	e, a
;_:	; Output byte loop
#macro PutCMainOutput
	rrc	e
	sbc	a, a
	out	(pLcdData), a
	out	(pLcdData), a
#endmacro
;	djnz	-_
	; Unroll loop
	PutCMainOutput
	PutCMainOutput
	PutCMainOutput
	PutCMainOutput
	PutCMainOutput
	PutCMainOutput
	PutCMainOutput
	PutCMainOutput
;	dec	c
;	jr	nz, PutMapOutLoop
	djnz	PutMapOutLoop
; Remainder
#if charWidth * charHeight != 126
	.error	"Fix the remainder portion of PutC."
#endif
	ld	a, (hl)
;	ld	b, 6
	xor	d	; inverse text support
	ld	e, a
;_:	; Output byte loop
;	rrc	e
;	sbc	a, a
;	out	(pLcdData), a
;	out	(pLcdData), a
;	djnz	-_
	PutCMainOutput
	PutCMainOutput
	PutCMainOutput
	PutCMainOutput
	PutCMainOutput
	PutCMainOutput
	call	AdvanceCursor
	popAndRestoreInterrupts
	pop	hl
	pop	de
	pop	bc
	pop	af
	ret


;------ ReadChar ---------------------------------------------------------------
ReadChar:
documentRoutine("ReadChar")
document("Reads the character code of the character currently under the cursor.")
document("WARNING: The LCD cursor is not reset!")
document("To advance to next character location, call SetCharCol AND SetCharRow to reset")
document("to previous value.  If you want to advance to the next location, call")
document("AdvanceCursor with textAutoScroll disabled.  (If it's enabled, calling")
document("AdvanceCursor could cause scrolling, which is probably not what you want since")
document("you're trying to READ non-destructively.)")
document("Inputs:")
document(" - Set LCD cursor to the correct location")
document("Output:")
document(" - A: 8-bit character code")
document("Destroys:")
document(" - LCD cursor")
	push	bc
	push	de
	push	hl
	pushAndDisableInterrupts
	ld	c, pLcdData
	ld	b, 8
	ld	a, lrGram
	out	(pLcdCmd), a
	out	(pLcdCmd), a
_:	in	a, (pLcdData)
	in	a, (pLcdData)
	in	h, (c)
	in	a, (pLcdData)
	ld	l, a
	xor	h
	add	a, a
	add	a, a
	add	a, a
	rr	e
	out	(c), h
	out	(c), l
	djnz	-_
	popAndRestoreInterrupts
	ld	a, e
	pop	hl
	pop	de
	pop	bc
	ret


;------ ResetScreen ------------------------------------------------------------
ResetScreen:
documentRoutine("ResetScreen")
document("Initalizes the screen modes and text control variables.  This must be the")
document("first thing you call for text mode.")
document("Inputs:")
document(" - None")
document("Output:")
document(" - Everything ready for text mode")
document("Destroys:")
document(" - I dunno, assume everything.")
	xor	a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	call	SetDirectionDown
	call	SetLcdWindowColumnBounds
	call	SetFullScrnWind
	jp	HomeUp


;------ FixCursor --------------------------------------------------------------
FixCursor:
documentRoutine("FixCursor")
document("Fixes up the LCD read/write cursor")
document("Inputs:")
document(" - None")
document("Output:")
document(" - LCD window set")
document("Destroys:")
document(" - AF")
	push	bc
	push	de
	push	hl
	call	SetCharCol
	call	SetCharRow
	pop	hl
	pop	de
	pop	bc
	ret


;------ SetCharRow -------------------------------------------------------------
SetCharRow:
documentRoutine("SetCharRow")
document("Sets the LCD window top and bottom for the current row of text.")
document("Inputs:")
document(" - None")
document("Output:")
document(" - LCD window set")
document("Destroys:")
document(" - AF")
document(" - BC")
document(" - E")
	; Setup
	pushAndDisableInterrupts
	ld	b, lrWinTop
	ld	c, pLcdCmd
	; Set row
	ld	a, (currentRow)
	; Multiply by 14
	add	a, a
	ld	e, a
	add	a, a
	add	a, a
	add	a, a
	sub	e
	; And write it
	out	(c), b
	out	(c), b
	inc	c
	out	(c), 0
	out	(pLcdData), a
	dec	c
	; lrWinRowEnd
	inc	b
	out	(c), b
	out	(c), b
	add	a, charHeight-1
	inc	c
	out	(c), 0
	out	(pLcdData), a
	dec	c
	; lrRow
	ld	b, lrRow
	out	(c), b
	out	(c), b
	sub	charHeight-1
	inc	c
	out	(c), 0
	out	(pLcdData), a
	dec	c
	popAndRestoreInterrupts
	ret


;------ SetCharCol -------------------------------------------------------------
SetCharCol:
documentRoutine("SetCharCol")
document("Sets the LCD write cursor column to match currentCol")
document("Input:")
document(" - None")
document("Output:")
document(" - LCD window set")
document("Destroys:")
document(" - AF")
document(" - C")
document(" - HL")
	; Set column
	pushAndDisableInterrupts
	ld	a, (currentCol)
	; Multiply by 9
	ld	h, a
	add	a, a
	add	a, a
	add	a, a
	add	a, h
	ld	l, a
	ld	a, h
	cp	29
	sbc	a, a
	inc	a
	ld	h, a
	ld	a, lrCol
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	ld	a, h
	out	(pLcdData), a
	ld	a, l
	out	(pLcdData), a
;	dec	c
	; lrWinColStart
;	ld	h, a
;	ld	de, charWidth
;	add	hl, de
	; lrWinColEnd
;	inc	b
;	out	(c), 0
;	out	(c), b
;	inc	c
;	out	(c), h
;	out	(c), l
	popAndRestoreInterrupts
	ret


;------ SetLcdWindowColumnBounds -----------------------------------------------
document("Sets the LCD window left/right bounds.")
document("These should be 0 and 314, because MicrOS rarely needs to keep writing to the")
document("same char location.")
document("Input:")
document(" - None")
document("Output:")
document(" - LCD left/right bounds reset")
document("Destroys:")
document(" - AF")
document("Assumes:")
document(" - Assumes that you want to be text mode.")
document("Execution Time:")
document(" - 134 cc not including inital CALL")
SetLcdWindowColumnBounds:
	pushAndDisableInterrupts
	xor	a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	ld	a, lrWinLeft
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	xor	a
	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdCmd), a
	ld	a, lrWinRight
	out	(pLcdCmd), a
	ld	a, 1
	out	(pLcdData), a
	ld	a, lcdLow(((charWidth * textCols) - 1))
	out	(pLcdData), a
	ld	a, lrWinBottom
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	xor	a
	out	(pLcdData), a
	ld	a, colorScrnHeight
	out	(pLcdData), a
	popAndRestoreInterrupts
	ret	;134 cc


;------ SetDirectionRight ------------------------------------------------------
SetDirectionRight:
documentRoutine("SetDirectionRight")
document("Sets the cursor to move left after every pixel.  At the end of the row, it")
document("then moves down;")
document("Inputs:")
document(" - None")
document("Output:")
document(" - Mode changed")
document("Destroys:")
document(" - AF")
document(" - H")
	pushAndDisableInterrupts
	push	hl
	ld	a, lrEntryMode
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	in	a, (pLcdData)
	ld	h, a
	in	a, (pLcdData)
	; Ignore the result
	ld	a, h
	out	(pLcdData), a
	ld	a, lcdCurMoveHoriz | lcdRowInc | lcdColInc
	out	(pLcdData), a
	pop	hl
	popAndRestoreInterrupts
	ret


;------ SetDirectionDown -------------------------------------------------------
SetDirectionDown:
documentRoutine("SetDirectionDown")
document("Sets the cursor to move down after every pixel.  At the end of the column, it")
document("then moves right.")
document("Inputs:")
document(" - None")
document("Output:")
document(" - Mode changed")
document("Destroys:")
document(" - AF")
document(" - H")
	pushAndDisableInterrupts
	ld	a, lrEntryMode
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	in	a, (pLcdData)
	ld	h, a
	in	a, (pLcdData)
	; Ignore the result
	ld	a, h
	out	(pLcdData), a
	ld	a, lcdRowInc | lcdColInc
	out	(pLcdData), a
	popAndRestoreInterrupts
	ret